#!/bin/bash
set -e

BZIP2_VERSION="1.0.8"
LIBMAD_VERSION="0.15.1b"
LIBJPEGTURBO_VERSION="3.1.3"
LIBPNG_VERSION="1.6.55"
LIBOGG_VERSION="1.3.6"
LIBVORBIS_VERSION="1.3.7"
FLAC_VERSION="1.5.0"
SDL3_VERSION="3.4.2"
BROTLI_VERSION="1.2.0"
HARFBUZZ_VERSION="12.3.2"
FREETYPE2_VERSION="2.14.1"
LIBDISCID_VERSION="0.6.5"
CJSON_VERSION="1.7.18"
ANCIENT_VERSION="2.3.0"
LIBICONV_VERSION="1.18"
GAMEMUSICEMU_VERSION="0.6.4"
UNIFONT_VERSION="17.0.03"
